﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace SolSoft.DataBinding.WpfDemo
{
	/// <summary>
	/// Simple class for implementing ICommand with delegates
	/// </summary>
	class DelegateCommand : ICommand
	{
		private readonly Func<object, bool> m_canExecute;
		private readonly Action<object> m_execute;

		public DelegateCommand(Func<object, bool> canExecute, Action<object> execute)
		{
			m_canExecute = canExecute;
			m_execute = execute;
		}

		public DelegateCommand(Func<bool> canExecute, Action execute)
		{
			//create closures that ignore the parameter
			m_canExecute = ignored => canExecute();
			m_execute = ignored => execute();
		}

		public event EventHandler CanExecuteChanged;
		protected virtual void OnCanExecuteChanged(EventArgs args)
		{
			CanExecuteChanged.Raise(this, args);
		}

		public bool CanExecute(object parameter)
		{
			return m_canExecute(parameter);
		}

		public void Execute(object parameter)
		{
			m_execute(parameter);
		}


		
	}
}
