﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SolSoft.DataBinding.WpfDemo
{
	/// <summary>
	/// Creates a delegate command whose 'can execute' state is tied to the given property
	/// Note that this creates a reference to that property (via the ValueChanged event) so the objects should have similar lifetimes
	/// Or use dispose
	/// </summary>
	class DelegateCommandFromProperty : DelegateCommand, IDisposable
	{
		private IProperty<bool> m_canExecuteProperty;

		public DelegateCommandFromProperty(IProperty<bool> canExecute, Action execute)
			: base(CreateCanExecuteDelegate(canExecute), execute)
		{
			m_canExecuteProperty = canExecute;

			//wire 'can execute' to the given boolean property
			m_canExecuteProperty.ValueChanged += canExecuteProperty_ValueChanged;
		}

		private static Func<bool> CreateCanExecuteDelegate(IProperty<bool> canExecute)
		{
			if (canExecute == null)
				throw new ArgumentNullException(ObjectNamingExtensions.GetName(() => canExecute));
			return () => canExecute.Value;
		}

		private void canExecuteProperty_ValueChanged(object sender, ValueChangedEventArgs e)
		{
			OnCanExecuteChanged(EventArgs.Empty);
		}


		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		protected virtual void Dispose(bool disposing)
		{
			if (disposing)
			{
				//dispose of managed resources:
				m_canExecuteProperty.ValueChanged -= canExecuteProperty_ValueChanged;
			}

			//dispose of unmanaged resources:
		}
	}
}
