﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Threading;

namespace SolSoft.DataBinding.WpfDemo
{
	/// <summary>
	/// See MSDN Magazine November 2015 "Async From The Start"
	/// https://msdn.microsoft.com/en-us/magazine/mt620013.aspx
	/// </summary>
	class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			App app = new App
			{
				ShutdownMode = ShutdownMode.OnExplicitShutdown
			};
			//app.InitializeComponent(); //doesn't exist right now
			SynchronizationContext.SetSynchronizationContext(new DispatcherSynchronizationContext());

			Program p = new Program();
			p.ExitRequested += (sender, e) =>
			{
				app.Shutdown();
			};

			Task programStart = p.StartAsync();
			HandleExceptions(programStart, app);

			app.Run();
		}

		private static async void HandleExceptions(Task task, Application wpfApplication)
		{
			try
			{
				await Task.Yield(); //ensure this runs as a continuation
				await task;
			}
			catch (Exception ex)
			{
				//deal with exception, either with message box
				//or delegating to general exception handling logic you may have wired up 
				//e.g. to app.DispatcherUnhandledException and AppDomain.UnhandledException
				MessageBox.Show(ex.ToString());

				wpfApplication.Shutdown();
			}
		}

		private Program()
		{
		}

		public Task StartAsync()
		{
			MainViewModel viewModel = new MainViewModel();
			viewModel.CloseRequested += viewModel_CloseRequested;

			EventHandler windowClosed = (sender, e) =>
			{
				viewModel.RequestClose();
			};


			MainWindow mainForm = new MainWindow();
			mainForm.Closed += windowClosed;
			mainForm.ViewModel = viewModel;
			mainForm.Show();

			//no async work to do in this case
			return Task.FromResult(default(object));
		}

		public event EventHandler<EventArgs> ExitRequested;
		void viewModel_CloseRequested(object sender, EventArgs e)
		{
			OnExitRequested(EventArgs.Empty);
		}

		protected virtual void OnExitRequested(EventArgs e)
		{
			ExitRequested.Raise(this, e);
		}
	}
}
